//+------------------------------------------------------------------+
//|                                                     MACD1D5m.mq5 |
//|                                                            Bruno |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Bruno"
#property link      "https://www.mql5.com"
#property version   "1.00"

//+------------------------------------------------------------------+
//|                                                  MACD Sample.mq5 |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright   "Copyright 2009-2013, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property version     "5.20"
#property description "It is important to make sure that the expert works with a normal"


//---
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
//---
input double InpLots          =0.01; // Lots

input color Resistance_Color=Red;
input ENUM_LINE_STYLE Resistance_Style;
input int Resistance_Width=1;
input color Support_Color=Red;
input ENUM_LINE_STYLE Support_Style;
input int Support_Width=1;
input int MinMax_nBar = 3;

//---
int ExtTimeOut=5; // time out in seconds between trade operations
//+------------------------------------------------------------------+
struct ChartPoint
{
   datetime time;
   double price;
};

struct StraightLineSpecification
{
   double m;
   double q;
   datetime start;
   datetime stop;
};
//+------------------------------------------------------------------+
class CSampleExpert
  {
protected:
   double            m_adjusted_point;             
   CTrade            m_trade;                      
   CSymbolInfo       m_symbol;                     
   CPositionInfo     m_position;                   
   CAccountInfo      m_account;                    
  

   //indicator handles
   int               m_handle_MinMaxM1;
   //--- indicator buffers
   double            m_tempMinM1[] ;
   double            m_tempMaxM1[];
   datetime          m_tempM1Time[];
   
   
private:
   //min max holder
   ChartPoint        m_MinM1[];
   ChartPoint        m_MaxM1[];
   
   //tempTrendPoint
   ChartPoint        m_tempTrendMax[];
   ChartPoint        m_tempTrendMin[];
   
   //coefficient of the trend lines
   StraightLineSpecification m_MaxLines[];
   StraightLineSpecification m_MinLines[];
   
   
   bool              searchMaxMin(int arrayElements ); 
   datetime          stop;
   datetime          start;
   datetime          constTime;
   int               copyTimeAndMaxMin(); 
   bool              preprocessing();
   bool              analyzeMaxMin();
   
   bool              twoPointsLine(ChartPoint &point1, ChartPoint &point2, double &m, double &q);
   double            distancePointLine(ChartPoint &point, double m, double q);
   double            averageDistancePointsLine(ChartPoint &array[], double m, double q);
   bool              fitLine(ChartPoint &array[], double &m, double &q);
   ChartPoint        pointInALine(double m, double q, datetime time);
public:
                     CSampleExpert(void);
                    ~CSampleExpert(void);
   bool              Init(void);
   void              Deinit(void);
   bool              Processing(void);

protected:
   bool              InitCheckParameters(const int digits_adjust);
   bool              InitIndicators(void);
   bool              LongClosed(void);
   bool              ShortClosed(void);
   bool              LongModified(void);
   bool              ShortModified(void);
   bool              LongOpened(void);
   bool              ShortOpened(void);
  };
//--- global expert
CSampleExpert ExtExpert;

//+------------------------------------------------------------------+
//| Create Arrow Up sign                                             |
//+------------------------------------------------------------------+
bool ArrowUpCreate(const long              chart_ID=0,           // chart's ID
                   const string            name="ArrowUp",       // sign name
                   const int               sub_window=0,         // subwindow index
                   datetime                time=0,               // anchor point time
                   double                  price=0,              // anchor point price
                   const ENUM_ARROW_ANCHOR anchor=ANCHOR_BOTTOM, // anchor type
                   const color             clr=clrRed,           // sign color
                   const ENUM_LINE_STYLE   style=STYLE_SOLID,    // border line style
                   const int               width=3,              // sign size
                   const bool              back=false,           // in the background
                   const bool              selection=true,       // highlight to move
                   const bool              hidden=true,          // hidden in the object list
                   const long              z_order=0)            // priority for mouse click
  {
//--- reset the error value
   ResetLastError();
//--- create the sign
   if(!ObjectCreate(chart_ID,name,OBJ_ARROW_UP,sub_window,time,price))
     {
      Print(__FUNCTION__,
            ": failed to create \"Arrow Up\" sign! Error code = ",GetLastError());
      return(false);
     }
//--- set anchor type
   ObjectSetInteger(chart_ID,name,OBJPROP_ANCHOR,anchor);
//--- set a sign color
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
//--- set the border line style
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,style);
//--- set the sign size
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,width);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
//--- enable (true) or disable (false) the mode of moving the sign by mouse
//--- when creating a graphical object using ObjectCreate function, the object cannot be
//--- highlighted and moved by default. Inside this method, selection parameter
//--- is true by default making it possible to highlight and move the object
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
//--- set the priority for receiving the event of a mouse click in the chart
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
  }

//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSampleExpert::CSampleExpert(void) : 
      m_adjusted_point(0),
      m_handle_MinMaxM1(INVALID_HANDLE),
      stop(-1),
      start(-1)                                   
  {
    ArraySetAsSeries(m_tempMaxM1, false);
    ArraySetAsSeries(m_tempMinM1, false);
    ArraySetAsSeries(m_tempM1Time, false);
    ArraySetAsSeries(m_MaxM1, false);
    ArraySetAsSeries(m_MinM1, false);
    ArraySetAsSeries(m_tempTrendMax, false);
    ArraySetAsSeries(m_tempTrendMin, false);
    ArraySetAsSeries(m_MaxLines, false);
    ArraySetAsSeries(m_MinLines, false);
    ArrayResize(m_MaxM1, 0, 1000);
    ArrayResize(m_MinM1, 0, 1000);
    ArrayResize(m_tempTrendMax, 0, 50);
    ArrayResize(m_tempTrendMin, 0, 50);
    ArrayResize(m_MaxLines, 0, 100);
    ArrayResize(m_MinLines, 0, 100);
    constTime = TimeCurrent();
    
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSampleExpert::~CSampleExpert(void)
  {
  }
//+------------------------------------------------------------------+
//| Initialization and checking for input parameters                 |
//+------------------------------------------------------------------+
bool CSampleExpert::Init(void)
  {
//--- initialize common information
   m_symbol.Name(Symbol());              // symbol
   m_trade.SetExpertMagicNumber(12345);  // magic
//--- tuning for 3 or 5 digits
   int digits_adjust=1;
   if(m_symbol.Digits()==3 || m_symbol.Digits()==5)
      digits_adjust=10;
   m_adjusted_point=m_symbol.Point()*digits_adjust;
//--- set default deviation for trading in adjusted points
   m_trade.SetDeviationInPoints(3*digits_adjust);
//---
  
   if(!InitCheckParameters(digits_adjust))
      return(false);
   if(!InitIndicators())
      return(false);
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking for input parameters                                    |
//+------------------------------------------------------------------+
bool CSampleExpert::InitCheckParameters(const int digits_adjust)
  {
//--- check for right lots amount
   if(InpLots<m_symbol.LotsMin() || InpLots>m_symbol.LotsMax())
     {
      printf("Lots amount must be in the range from %f to %f",m_symbol.LotsMin(),m_symbol.LotsMax());
      return(false);
     }
   if(MathAbs(InpLots/m_symbol.LotsStep()-MathRound(InpLots/m_symbol.LotsStep()))>1.0E-10)
     {
      printf("Lots amount is not corresponding with lot step %f",m_symbol.LotsStep());
      return(false);
     }
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialization of the indicators                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::InitIndicators(void)
  {
//--- create  indicator
   if(m_handle_MinMaxM1==INVALID_HANDLE)
      if((m_handle_MinMaxM1=iCustom(NULL, PERIOD_M1, "MyIndicators\\MaxMin.ex5", MinMax_nBar) )==INVALID_HANDLE)
      //if((m_handle_MinMaxM1=iFractals(NULL, PERIOD_M1))==INVALID_HANDLE)
        {
         printf("Error creating MinMax indicator");
         return(false);
        }

//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//| Check for long position closing                                  |
//+------------------------------------------------------------------+
bool CSampleExpert::LongClosed(void)
  {
   bool res=false;
//--- should it be closed?
            if( true )
            {
               
               //--- close position
               if(m_trade.PositionClose(Symbol()))
                  printf("Long position by %s to be closed",Symbol());
               else
                  printf("Error closing position by %s : '%s'",Symbol(),m_trade.ResultComment());
               //--- processed and cannot be modified
               res=true;
            }

   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position closing                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::ShortClosed(void)
  {
   bool res=false;
//--- should it be closed?
  if(true  )
            {
               //--- close position
               if(m_trade.PositionClose(Symbol()))
                  printf("Short position by %s to be closed",Symbol());
               else
                  printf("Error closing position by %s : '%s'",Symbol(),m_trade.ResultComment());
               //--- processed and cannot be modified
               res=true;
            }
             
            
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for long position modifying                                |
//+------------------------------------------------------------------+
bool CSampleExpert::LongModified(void)
  {
   bool res=false;
  /* 
  if(m_position.PriceCurrent - m_position.Pri)
        {
         double sl=0.0;
         double tp=m_position.PriceOpen();
         
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Long position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           
        }
    else if(m_macd5m_current > m_signal5m_current)
        {
         double sl=0.0;
         double tp=0.0;
         
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Long position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           
        }
    
//--- result
*/
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position modifying                               |
//+------------------------------------------------------------------+
bool CSampleExpert::ShortModified(void)
  { bool res=false;
  
  /* if(m_position.Profit() < 0 && m_macd5m_current > m_signal5m_current)
        {
         double sl=0.0;
         double tp=m_position.PriceOpen();
         
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Short position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           
        }
   else if(m_macd5m_current < m_signal5m_current)
        {
         double sl=0.0;
         double tp=0.0;
         
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Short position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           
        }
    */ 
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for long position opening                                  |
//+------------------------------------------------------------------+
bool CSampleExpert::LongOpened(void)
  {
   bool res=false;
//--- check for long position (BUY) possibility
      if(true)
      {
         double price=m_symbol.Ask();
         double tp   =0.;
         double sl   =0.;
        double lots = 2 *  InpLots;
         
         //--- check for free money
         if(m_account.FreeMarginCheck(Symbol(),ORDER_TYPE_BUY,lots,price)<0.0)
            printf("We have no money. Free Margin = %f",m_account.FreeMargin());
         else
         {
            //--- open position
            if(m_trade.PositionOpen(Symbol(),ORDER_TYPE_BUY,lots,price,sl,tp))
               printf("Position by %s to be opened: price=%f,TP=%f, SL=%f",Symbol(),price, tp, sl);
            else
            {
               printf("Error opening BUY position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Open parameters : price=%f,TP=%f, SL=%f",price, tp, sl);
            }
         }
         //--- in any case we must exit from expert
         res=true;
      }
      
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position opening                                 |
//+------------------------------------------------------------------+
bool CSampleExpert::ShortOpened(void)
  {
   bool res=false;
//--- check for short position (SELL) possibility
   if(true) 
   {
      double price=m_symbol.Bid();
      double tp   = 0.;
      double sl   = 0.;
      double lots = 2 *  InpLots;
      Print("vendo " ,lots);
            //--- check for free money
      if(m_account.FreeMarginCheck(Symbol(),ORDER_TYPE_SELL,lots,price)<0.0)
         printf("We have no money. Free Margin = %f",m_account.FreeMargin());
      else
      { //--- open position
         if(m_trade.PositionOpen(Symbol(),ORDER_TYPE_SELL,lots,price,sl,tp))
         printf("Position by %s to be opened: price=%f,TP=%f, SL=%f",Symbol(),price, tp, sl);
         else
         {
            printf("Error opening BUY position by %s : '%s'",Symbol(),m_trade.ResultComment());
            printf("Open parameters : price=%f,TP=%f, SL=%f",price, tp, sl);
         }
      }
            //--- in any case we must exit from expert
      res=true;
    }
//--- result
   return(res);
}

//+------------------------------------------------------------------+
//| Create a trend line by the given coordinates                     |
//+------------------------------------------------------------------+
bool TrendCreate(const long            chart_ID=0,        // chart's ID
                 const string          name="TrendLine",  // line name
                 const int             sub_window=0,      // subwindow index
                 datetime              time1=0,           // first point time
                 double                price1=0,          // first point price
                 datetime              time2=0,           // second point time
                 double                price2=0,          // second point price
                 const color           clr=clrRed,        // line color
                 const ENUM_LINE_STYLE style=STYLE_SOLID, // line style
                 const int             width=1,           // line width
                 const bool            back=false,        // in the background
                 const bool            selection=true,    // highlight to move
                 const bool            ray_left=false,    // line's continuation to the left
                 const bool            ray_right=false,   // line's continuation to the right
                 const bool            hidden=true,       // hidden in the object list
                 const long            z_order=0)         // priority for mouse click
  {
//--- set anchor points' coordinates if they are not set
   ChangeTrendEmptyPoints(time1,price1,time2,price2);
//--- reset the error value
   ResetLastError();
//--- create a trend line by the given coordinates
   if(!ObjectCreate(chart_ID,name,OBJ_TREND,sub_window,time1,price1,time2,price2))
     {
      Print(__FUNCTION__,
            ": failed to create a trend line! Error code = ",GetLastError());
      return(false);
     }
//--- set line color
   ObjectSetInteger(chart_ID,name,OBJPROP_COLOR,clr);
//--- set line display style
   ObjectSetInteger(chart_ID,name,OBJPROP_STYLE,style);
//--- set line width
   ObjectSetInteger(chart_ID,name,OBJPROP_WIDTH,width);
//--- display in the foreground (false) or background (true)
   ObjectSetInteger(chart_ID,name,OBJPROP_BACK,back);
//--- enable (true) or disable (false) the mode of moving the line by mouse
//--- when creating a graphical object using ObjectCreate function, the object cannot be
//--- highlighted and moved by default. Inside this method, selection parameter
//--- is true by default making it possible to highlight and move the object
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTABLE,selection);
   ObjectSetInteger(chart_ID,name,OBJPROP_SELECTED,selection);
//--- enable (true) or disable (false) the mode of continuation of the line's display to the left
   ObjectSetInteger(chart_ID,name,OBJPROP_RAY_LEFT,ray_left);
//--- enable (true) or disable (false) the mode of continuation of the line's display to the right
   ObjectSetInteger(chart_ID,name,OBJPROP_RAY_RIGHT,ray_right);
//--- hide (true) or display (false) graphical object name in the object list
   ObjectSetInteger(chart_ID,name,OBJPROP_HIDDEN,hidden);
//--- set the priority for receiving the event of a mouse click in the chart
   ObjectSetInteger(chart_ID,name,OBJPROP_ZORDER,z_order);
//--- successful execution
   return(true);
  }
//+------------------------------------------------------------------+
//| Move trend line anchor point                                     |
//+------------------------------------------------------------------+
bool TrendPointChange(const long   chart_ID=0,       // chart's ID
                      const string name="TrendLine", // line name
                      const int    point_index=0,    // anchor point index
                      datetime     time=0,           // anchor point time coordinate
                      double       price=0)          // anchor point price coordinate
  {
//--- if point position is not set, move it to the current bar having Bid price
   if(!time)
      time=TimeCurrent();
   if(!price)
      price=SymbolInfoDouble(Symbol(),SYMBOL_BID);
//--- reset the error value
   ResetLastError();
//--- move trend line's anchor point
   if(!ObjectMove(chart_ID,name,point_index,time,price))
     {
      Print(__FUNCTION__,
            ": failed to move the anchor point! Error code = ",GetLastError());
      return(false);
     }
//--- successful execution
   return(true);
  }
//+------------------------------------------------------------------+
//| The function deletes the trend line from the chart.              |
//+------------------------------------------------------------------+
bool TrendDelete(const long   chart_ID=0,       // chart's ID
                 const string name="TrendLine") // line name
  {
//--- reset the error value
   ResetLastError();
//--- delete a trend line
   if(!ObjectDelete(chart_ID,name))
     {
      Print(__FUNCTION__,
            ": failed to delete a trend line! Error code = ",GetLastError());
      return(false);
     }
//--- successful execution
   return(true);
  }
//+------------------------------------------------------------------+
//| Check the values of trend line's anchor points and set default   |
//| values for empty ones                                            |
//+------------------------------------------------------------------+
void ChangeTrendEmptyPoints(datetime &time1,double &price1,
                            datetime &time2,double &price2)
  {
//--- if the first point's time is not set, it will be on the current bar
   if(!time1)
      time1=TimeCurrent();
//--- if the first point's price is not set, it will have Bid value
   if(!price1)
      price1=SymbolInfoDouble(Symbol(),SYMBOL_BID);
//--- if the second point's time is not set, it is located 9 bars left from the second one
   if(!time2)
     {
      //--- array for receiving the open time of the last 10 bars
      datetime temp[10];
      CopyTime(Symbol(),Period(),time1,10,temp);
      //--- set the second point 9 bars left from the first one
      time2=temp[0];
     }
//--- if the second point's price is not set, it is equal to the first point's one
   if(!price2)
      price2=price1;
  }
//+------------------------------------------------------------------+
//| main function returns true if any position processed             |
//+------------------------------------------------------------------+
bool CSampleExpert::Processing(void)
  {
  //la prima volta copio tutti i valori relativi all'ultimo mese e li analizzo
  //le altre volte scaricher soltanto i dati nuovi
  //creo array con elenco dei minimi e dei massimi
  //ad ogni ciclo controllo se appaiono di nuovi
  
  //considero i massimi, prendo gli ultimi 2 e ricavo una retta, 
  //poi prendo il massimo ancora precedente e vedo se si scosta poco dalla precedente retta
  //se si scosta poco ricavo una retta di fit lineare e considero il massimo ancora precedente
  //mi fermo quando trovo uno scostamento "grande" (la tolleranza deve essere grande quando 
  //la retta  formata da due soli punti e pi piccola man mano che aumentano i punti di fit)
  //ripeto la stessa procedura per i minimi
  //successivamente, ad ogni nuovo max/min si controlla se sta bene nella retta di fit. Se sta bene ok. 
  //Se no si considera una nuova retta tra il nuovo max e quello precedente e si attendono i nuovi max.
  //stessa cosa per i minimi
  
  
//--- refresh rates
      if(!m_symbol.RefreshRates())
         return(false);
      
      //searching maximum e minimum and filling the m_MaxM1 and m_MinM1 arrays
      if(!preprocessing()) 
         return false;
      
      //start to analyze max and min to find trendline
      analyzeMaxMin();  
      //m_MaxLines and m_MinLines are now updated
      
      //plotting an arrow at each max 
      {
         static int size=0;
         int prevSize = 0;
         if(ArraySize(m_MaxM1) > size){
            prevSize = size;
            size = ArraySize(m_MaxM1);
            for(int i=prevSize; i < size; i++){
               ObjectCreate(0,StringFormat("ArrowUp%d", i),OBJ_ARROW,0,m_MaxM1[i].time,m_MaxM1[i].price+0.00015);
               ObjectSetInteger(0,StringFormat("ArrowUp%d", i),OBJPROP_ARROWCODE,217);               
            }
         }
      }
      
      //plotting an arrow at each min 
      {
         static int size=0;
         int prevSize = 0;
         if(ArraySize(m_MinM1) > size){
            prevSize = size;
            size = ArraySize(m_MinM1);
            for(int i=prevSize; i < size; i++){
               ObjectCreate(0,StringFormat("ArrowDown%d", i),OBJ_ARROW,0,m_MinM1[i].time,m_MinM1[i].price-0.00005);
               ObjectSetInteger(0,StringFormat("ArrowDown%d", i),OBJPROP_ARROWCODE,218);               
            }
         }
      }
      
      //Plotting m_MaxLines and m_MinLines
     // Print ("Plotting trendlines for max...");
      for(int i = 0; i < ArraySize(m_MaxLines); i++){
         if(ObjectFind(0, StringFormat("max trendline%d", i)))
            TrendDelete(0,StringFormat("max trendline%d", i));
         ChartPoint point1, point2;
         point1 = pointInALine(m_MaxLines[i].m, m_MaxLines[i].q, m_MaxLines[i].start-constTime);
         point2 = pointInALine(m_MaxLines[i].m, m_MaxLines[i].q, m_MaxLines[i].stop-constTime);
         if(!TrendCreate(0, StringFormat("max trendline%d", i), 0, point1.time+constTime, point1.price,
                           point2.time+constTime, point2.price, C'255,0,0' , STYLE_SOLID,
                            1, false, false, false,false))
              return false;   
      }
      ChartRedraw();
  /*    
   //comparing price and trendline
   if(ObjectGetValueByTime(0, "max trend line", TimeCurrent() ) <  m_symbol.Ask()){
      Print("Now up to the max trend line");
   }
   if(ObjectGetValueByTime(0, "max trend line", TimeCurrent() ) >  m_symbol.Ask()){
      Print("Now down to the max trend line");
   }
   if(ObjectGetValueByTime(0, "min trend line", TimeCurrent() ) <  m_symbol.Ask()){
      Print("Now up to the min trend line");
   }
   if(ObjectGetValueByTime(0, "min trend line", TimeCurrent() ) >  m_symbol.Ask()){
      Print("Now down to the min trend line");
   }          
  
  */
  
//--- it is important to enter the market correctly, 
//--- but it is more important to exit it correctly...   
//--- first check if position exists - try to select it
 /*  if(m_position.Select(Symbol()))
     {
      if(m_position.PositionType()==POSITION_TYPE_BUY)
        {
         //--- try to close or modify long position
         if(LongClosed())
            return(true);
         if(LongModified())
            return(true);
        }
      else
        {
         //--- try to close or modify short position
         if(ShortClosed())
            return(true);
         if(ShortModified())
            return(true);
        }
     }
//--- no opened position identified
   else
     {
      //--- check for long position (BUY) possibility
      if(LongOpened())
         return(true);
      //--- check for short position (SELL) possibility
      if(ShortOpened())
         return(true);
     }*/
//--- exit without position processing
   return(false);
  }

bool CSampleExpert::preprocessing(){
      static bool firstTime = true;
      if(firstTime){         //is the first time operating
         //searching the previous bar
         //Print ("is first time operating...");
         datetime previousBar[1] ;
         int copied = CopyTime(NULL, PERIOD_M1,1,1, previousBar);
         if(copied != 1) {
            //Print("Error in coping prevBar time");
            return false; 
            }
         int month = 60*60*10; 
         start = previousBar[0];
         stop = previousBar[0] - month;        
         firstTime = false;
      }
      else{ //check for other new maximum or minimum
         //Print ("is not the first time operating...");
         datetime previousBar[1] ;
         int copied = CopyTime(NULL, PERIOD_M1,1,1, previousBar);
         if(copied != 1 || previousBar[0] == start) {
           // Print("Error in coping prevBar time");
            return false; }
         start = previousBar[0];
      }  
      
     // Print ("Coping max min and time...");
      int copySize=0;
      copySize = copyTimeAndMaxMin();
      if(copySize <=0 ) return false;         
      Print("start:",start," stop:",stop, " copysize:", copySize);
      
      //searching maximum e minimum and filling the m_MaxM1 and m_MinM1 arrays
    //  Print ("Searching max and min");
      searchMaxMin(copySize);

     //finish up to start
     stop = start;
     return true;
}

//returns the number of bars copied, -1 for errors
int CSampleExpert::copyTimeAndMaxMin()
{
   ArrayFree(m_tempMaxM1);
   ArrayFree(m_tempMinM1);
   ArrayFree(m_tempM1Time);
   
   
   
   ResetLastError();
   int copied1 = CopyBuffer(m_handle_MinMaxM1,0,start, stop, m_tempMaxM1);
   if(copied1<=0)
   {
      Print("Unable to copy the maximums. Error = ",GetLastError());
      return -1;
   }
   for(int i=0; i< copied1; i++)
   {
      if(m_tempMaxM1[i]<0.0001) {
         Print("no data ready");
         return -1;
      }
   }

   ResetLastError();
   int copied2 = CopyBuffer(m_handle_MinMaxM1,1,start, stop, m_tempMinM1);
   if(copied2 != copied1)
   {
      Print("Unable to copy the minimums. Error = ",GetLastError());
      return -1;
   }
   
   for(int i=0; i< copied2; i++)
   {
      if(m_tempMinM1[i]<0.0001) {
         Print("no data ready");
         return -1;
      }
   }
      
   ResetLastError();
   int copied3 = CopyTime(NULL, PERIOD_M1,start, stop, m_tempM1Time);
   if(copied3 != copied2)
   {
      Print("Unable to copy the times. Error = ",GetLastError());
      return -1;
   }
   
   for(int i=0; i< copied3; i++)
   {
      if(m_tempM1Time[i]<0.0001) {
         Print("no data ready");
         return -1;
      }
   }
   return copied3;
   
}
  
/*
Search max and minimum in m_tempmaxM1 and m_tempminM1 and put the datetime value in m_MinM1 and m_MaxM1
*/
bool CSampleExpert::searchMaxMin(int arrayElements)
{
      
      for(int i=0; i < arrayElements; i++)
      {
        // Print("m_tempMaxM1[", i,"]=",m_tempMaxM1[i]);
         if(m_tempMaxM1[i] != EMPTY_VALUE && m_tempMaxM1[i] != 0.0){ //max found
            int prevSize = ArraySize(m_MaxM1);
            ArrayResize(m_MaxM1, prevSize + 1, 1000 );
            m_MaxM1[prevSize].price = m_tempMaxM1[i];
            m_MaxM1[prevSize].time  = m_tempM1Time[i]; 
            Print("Max found at: ", m_MaxM1[prevSize].time, "   ", m_MaxM1[prevSize].price);              
         }
         //Print("m_tempMinM1[", i,"]=",m_tempMinM1[i]);
         if(m_tempMinM1[i] != EMPTY_VALUE && m_tempMinM1[i] != 0.0){ //min found
            int prevSize = ArraySize(m_MinM1);
            ArrayResize(m_MinM1, prevSize + 1, 1000 );
            m_MinM1[prevSize].price = m_tempMinM1[i];
            m_MinM1[prevSize].time  = m_tempM1Time[i];
            Print("Min found at: ", m_MinM1[prevSize].time, "   ", m_MinM1[prevSize].price);
         }      
      }
      //success 
      
      return true;
}

bool CSampleExpert::analyzeMaxMin()
{
   
   static bool firstTime = true;
   static int linesMaxIndex = 0;
   if(firstTime){
      //take the last two max and make a trendline. Then take the previous max and look if it
      //fit the trendline. If yes, good and calculate another trendline the is the best fit of hte maximums. 
      //If not, stop.
      //the same for min. 
      //at the end return the straight line coefficients
      //firstTimeAnalisysForMax();
      //take the last two max and calculate a straight line equation
      if(ArraySize(m_MaxM1) < 2) 
         return false;
      ArraySetAsSeries(m_MaxM1, true);
      ArrayResize(m_tempTrendMax, 2, 50);
      m_tempTrendMax[0] = m_MaxM1[0];
      m_tempTrendMax[1] = m_MaxM1[1];
      ArrayResize(m_MaxLines, 1, 100);
      twoPointsLine(m_tempTrendMax[0], m_tempTrendMax[1], m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);
      m_MaxLines[linesMaxIndex].start = m_tempTrendMax[1].time;
      m_MaxLines[linesMaxIndex].stop = m_tempTrendMax[0].time;
      for (int i=2; i< ArraySize(m_MaxM1); i++){
         double distance = distancePointLine(m_MaxM1[i], m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);
         distance = NormalizeDouble(distance, 8);
         //double avgDistance = averageDistancePointsLine(m_tempTrendMax, m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);  
       //  Print("Distance ", distance);
         if(distance > 0.0005 ){ //not in the same trendline
            Print("TrendLine found of ", i, " points.");
            break;
         }
         else{ //we have to consider this point in the trendline, so we calculate a new trendline 
               //that is a linear fit of the previous point and this point too
            ArrayResize(m_tempTrendMax, i + 1, 50);
            m_tempTrendMax[i]= m_MaxM1[i];
            fitLine(m_tempTrendMax, m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);
            m_MaxLines[linesMaxIndex].start = m_tempTrendMax[i].time;
         }
      }
      ArraySetAsSeries(m_MaxM1, false);
      //firstTimeAnalisysForMin();
      firstTime = false;
   }
   else{//not first time operating
      //AnalisysForMax();
      
      if(m_MaxM1[0].time > m_MaxLines[linesMaxIndex].stop) {//we have another maximum
         Print("Found another new maximum");
         ArraySetAsSeries(m_MaxM1, true);
         double distance = distancePointLine(m_MaxM1[0], m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);
         //double avgDistance = averageDistancePointsLine(m_tempTrendMax, m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);  
         //Print("Distance ", distance, "    avgDistance ", avgDistance);
         if(distance >= 0.0005){ //not in the same trendline
                                     //we start another one
            Print("Starting another trendline");
            linesMaxIndex++;
            ArrayResize(m_tempTrendMax, 2, 50);   
            m_tempTrendMax[0] = m_MaxM1[1];
            m_tempTrendMax[1] = m_MaxM1[0];
            ArrayResize(m_MaxLines, linesMaxIndex+1, 100);
            twoPointsLine(m_tempTrendMax[0], m_tempTrendMax[1], m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);
            m_MaxLines[linesMaxIndex].start = m_tempTrendMax[0].time;
            m_MaxLines[linesMaxIndex].stop = m_tempTrendMax[1].time;
         }
         else{ //we have to consider this point in the trendline, so we calculate a new trendline 
               //that is a linear fit of the previous point and this point too
            Print("Found another point of the previous trendline");
            ArrayResize(m_tempTrendMax, ArraySize(m_tempTrendMax) + 1, 50);
            m_tempTrendMax[ArraySize(m_tempTrendMax) - 1] = m_MaxM1[0];
            fitLine(m_tempTrendMax, m_MaxLines[linesMaxIndex].m, m_MaxLines[linesMaxIndex].q);
            m_MaxLines[linesMaxIndex].stop = m_tempTrendMax[ArraySize(m_tempTrendMax) - 1].time;
         }
         ArraySetAsSeries(m_MaxM1, false);
      }
      
      //AnalisysForMin();
   }
   return true;
}

bool   CSampleExpert::twoPointsLine(ChartPoint &point1, ChartPoint &point2, double &m, double &q)
{
   //return a line for the point point1 and point2
   //false for a vertical line
   if(point2.time - point1.time == 0) return false;
   m = (point2.price - point1.price) / (point2.time - point1.time);
   q = NormalizeDouble(- m * (point1.time-constTime) + point1.price,8);
   return true;
}

double            CSampleExpert::distancePointLine(ChartPoint &point, double m, double q)
{
   //return the distance for point to line y= mx + q
   return MathAbs(point.price - (m*(point.time - constTime) + q)) / MathSqrt(1 + m * m);
}

double            CSampleExpert::averageDistancePointsLine(ChartPoint &array[], double m, double q)
{
   //return the average distance between the points of the array and the line y = mx + q
   double distance = 0;
   if(ArraySize(array) <= 2) return 0.0;
   
   for(int i = 0; i< ArraySize(array) -1; i++)
   {
      distance += distancePointLine(array[i], m, q);
   }
   return distance/ArraySize(array);
}

bool              CSampleExpert::fitLine(ChartPoint &array[], double &m, double &q)
{
//put in m and q the coefficients for the fit line of all the points in array
//http://www3.unict.it/fisfarm/spiegazioni/BEST%20FIT%20LINEARE.htm
//y=mx + q
//m=(nA-BC)/(nD-B^2)            q=(CD-AB)/(nD-B^2)
//A=S[ Xi Yi];        B=S[ Xi];       C=S[ Yi];       D =S[ Xi^2]
//S = sum over i

   double A, C;
   double B, D;
   A=0; B=0; C=0; D=0;
   int n = ArraySize(array);
   for(int i = 0; i< n; i++){
      double time=NormalizeDouble((double)(array[i].time-constTime),8);
      A+=array[i].price * time;
      B+=time;
      C+=array[i].price;
      D+=time*time;
   }
   if((n*D - B*B) == 0){      //Da sistemare!!!!!!!!!!!!!!!!!!!!!!!!!!! 
      Print("Error in fitting the points");
      return false;
   }
  // Print("A=", A," B=",B," C=",C," D=",D);
   m=NormalizeDouble((n * A - B*C) / (n*D - B*B),8);
   q=NormalizeDouble((C * D - A*B)/(n*D-B*B),8);
   return true;  
}
/*
bool              CSampleExpert::fitLine(ChartPoint &array[], double &m, double &q)
{
//put in m and q the coefficients for the fit line of all the points in array
//http://www3.unict.it/fisfarm/spiegazioni/BEST%20FIT%20LINEARE.htm
//y=mx + q
//m=(nA-BC)/(nD-B^2)            q=(CD-AB)/(nD-B^2)
//A=S[ Xi Yi];        B=S[ Xi];       C=S[ Yi];       D =S[ Xi^2]
//S = sum over i

   double A, C;
   double B, D;
   A=0; B=0; C=0; D=0;
   int n = ArraySize(array);
   double constant=0;//NormalizeDouble((double)array[n-1].time,8);
   for(int i = 0; i< n; i++){
      double time=NormalizeDouble((double)(array[i].time-constant),8);
      Print("time:",time);
      A+=array[i].price * time;
      B+=time;
      C+=array[i].price;
      D+=time*time;
   }
   if((n*D - B*B) == 0){      //Da sistemare!!!!!!!!!!!!!!!!!!!!!!!!!!! 
      Print("Error in fitting the points");
      return false;
   }
   Print("A=", A," B=",B," C=",C," D=",D);
   m=NormalizeDouble((n * A - B*C)/(n*D-B*B),8);
  // A*=n*constant;
  // B*=n*constant;
  // D+=2*B+n*constant*constant;
   
   q=NormalizeDouble((C * D - A*B)/(n*D-B*B),8);
   return true;  
}
*/
ChartPoint        CSampleExpert::pointInALine(double m, double q, datetime time)
{  //return the point of the line y=mx+q at x=time
   ChartPoint point;
   point.time=time;
   point.price=NormalizeDouble(m*time + q,8);
   return point;
}
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- create timer
   EventSetTimer(60);
      
//---
//--- create all necessary objects
   if(!ExtExpert.Init())
      return(INIT_FAILED);
//--- secceed
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
   EventKillTimer();
      
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
  
   static datetime limit_time=0; // last trade processing time + timeout
//--- don't process if timeout
   if(TimeCurrent()>=limit_time)
     {
      //--- check for data
      if(Bars(Symbol(),Period())>2*26)
        {
        
         //--- change limit time by timeout in seconds if processed
         ExtExpert.Processing();
         
            limit_time=TimeCurrent()+ExtTimeOut;
            
          
        }
     }
//---
   
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Trade function                                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| TradeTransaction function                                        |
//+------------------------------------------------------------------+
void OnTradeTransaction(const MqlTradeTransaction& trans,
                        const MqlTradeRequest& request,
                        const MqlTradeResult& result)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester()
  {
//---
   double ret=0.0;
//---

//---
   return(ret);
  }
//+------------------------------------------------------------------+
//| TesterInit function                                              |
//+------------------------------------------------------------------+
void OnTesterInit()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| TesterPass function                                              |
//+------------------------------------------------------------------+
void OnTesterPass()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| TesterDeinit function                                            |
//+------------------------------------------------------------------+
void OnTesterDeinit()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| BookEvent function                                               |
//+------------------------------------------------------------------+
void OnBookEvent(const string &symbol)
  {
//---
   
  }
//+------------------------------------------------------------------+
